# Generated by Django 5.0.4 on 2024-04-16 11:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advance', '0011_rename_account_global_api_key_cloudflare_api_global_api_key'),
    ]

    operations = [
        migrations.AddField(
            model_name='cloudflare_api',
            name='ip_address',
            field=models.GenericIPAddressField(default='127.0.0.1'),
        ),
        migrations.AddField(
            model_name='cloudflare_api',
            name='ip_address2',
            field=models.GenericIPAddressField(default='127.0.0.2'),
        ),
    ]



from django.urls import path
from bigsansar.contrib.advance import views


urlpatterns = [
    path('', views.cloudflare),
    path('api_key', views.cf_conf, name='cloudflare api configrations'),
    path('hostname', views.cf_main_domain, name = 'main server domain'),
    path('ssl_origin', views.ssl_origin, name='ssl_origin'),
    # path('a', views.ff)
]


# Generated by Django 5.0.2 on 2024-02-12 05:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_default_domain'),
    ]

    operations = [
        migrations.AddField(
            model_name='domains',
            name='default',
            field=models.BooleanField(default='False'),
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-10 05:16

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advance', '0009_sidebarsettings_user'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='cloudflare_api',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('account_global_api_key', models.CharField(max_length=500)),
                ('email', models.EmailField(max_length=254)),
                ('main_domain_name', models.CharField(max_length=100)),
                ('account_id', models.CharField(max_length=500)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 5.0.4 on 2024-05-09 10:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0010_alter_domains_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='domains',
            name='zone_id',
            field=models.CharField(default='', max_length=1000),
        ),
    ]



# Generated by Django 4.2 on 2023-06-18 11:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('advance', '0004_alter_javascript_options'),
    ]

    operations = [
        migrations.DeleteModel(
            name='css',
        ),
    ]



# Generated by Django 5.0.4 on 2024-04-13 11:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('advance', '0010_cloudflare_api'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cloudflare_api',
            old_name='account_global_api_key',
            new_name='global_api_key',
        ),
    ]



# Generated by Django 4.2 on 2023-05-13 03:08

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('advance', '0002_alter_css_options'),
    ]

    operations = [
        migrations.CreateModel(
            name='javascript',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('javascript', models.TextField()),
                ('domain', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='sites.domains')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Javascripts',
                'verbose_name_plural': 'Javascripts',
            },
        ),
    ]



# Generated by Django 5.0.2 on 2024-02-12 05:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0003_domains_default'),
    ]

    operations = [
        migrations.AlterField(
            model_name='domains',
            name='default',
            field=models.BooleanField(default=False),
        ),
    ]



# from django.contrib import admin
# from django.contrib.admin import SimpleListFilter
# from django.utils.translation import gettext_lazy as _
# from bigsansar.contrib.sites.forms import create_domainform, customaddpageform, custompageform
# from bigsansar.contrib.sites.models import default_domain, domains, pages
# from django.contrib.auth.models import User


# # Register your models here.


# class domain_filter(SimpleListFilter):
#     title = _('Domains')  # a label for our filter
#     parameter_name = "domain"  # you can put anything here

#     def lookups(self, request, model_admin):
#         # This is where you create filter options; we have two:
#         qs = model_admin.get_queryset(request)
#         return qs.values_list('domain_id', 'domain__domain').order_by('domain').distinct()

#     def queryset(self, request, queryset):

#         # This is where you process parameters selected by use via filter options:
#         if self.value():
#             return queryset.filter(domain=self.value())




# class pageadmin(admin.ModelAdmin):
#     add_form = customaddpageform
#     form = custompageform
#     prepopulated_fields = {"slug": ("title",)}
#     list_display = ['slug', 'domain', 'publish_date', 'visitor']
#     list_filter = (domain_filter,)
#     search_fields = ['slug']

#     def get_form(self, request, obj=None, **kwargs):
#         """
#         Use special form during foo creation
#         """
#         defaults = {}
#         if obj is None:
#             defaults['form'] = self.add_form
#         defaults.update(kwargs)
#         return super().get_form(request, obj, **defaults)
        


# admin.site.register(pages, pageadmin)

# admin.site.register(domains)



# Generated by Django 5.0.2 on 2024-03-17 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('advance', '0007_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SidebarSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_sidebar_open', models.BooleanField(default=True)),
            ],
        ),
    ]



import json
from django.http import HttpResponse, JsonResponse
from django.shortcuts import redirect, render
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth import login, logout, update_session_auth_hash, authenticate
from django.core.paginator import Paginator
import requests
from bigsansar.contrib.advance.forms import cf_api_key, cf_hostname, chpassform, custom_admin_upadte, loginform, profileform, userchform, usrinfoform
from bigsansar.contrib.advance.models import admin_update, cloudflare_api
from bigsansar.contrib.sites.models import default_domain, domains
from django.contrib import messages

from www.settings import BASE_DIR

# Create your views here.
def admin_redirect(request):
    return redirect('admin/')


@csrf_exempt
def userlogin(request):
    if request.user.is_authenticated:

        if request.user.is_staff:
           
                    try:
                        get_default_domain = default_domain.objects.get(user=request.user)
                    except:
                        return redirect('/admin/domain/create')
                    else:

                        url_get = "/admin/domain/manage/%s" % (get_default_domain.domain.id)
                        return redirect(url_get)
        else:
            return render(request, '404.html')
 
    else:
        if request.method == "POST":
            fm = loginform(request=request, data=request.POST)
            if fm.is_valid():
                uname = fm.cleaned_data['username']
                upass = fm.cleaned_data['password']
                user = authenticate(username=uname, password=upass)
                if user is not None and user.is_staff:
                    login(request, user)
                    try:
                        get_default_domain = default_domain.objects.get(user=request.user)
                    except:
                        return redirect('/admin/domain/create')
                    else:

                        url_get = "/admin/domain/manage/%s" % (get_default_domain.domain.id)
                        return redirect(url_get)
                
                else:
                    messages.error(request, "Invalid credentials or you do not have staff access.")

        else:
            fm = loginform()
        return render(request, 'admin/login.html', {'form': fm})
    

def userlogout(request):
    logout(request)
    return redirect('/admin')


def account(request):
    if request.user.is_authenticated:

        if request.user.is_staff:
            user = request.user
            getdomain = domains.objects.filter(user=user).order_by('-id')
            paginator = Paginator(getdomain, 5)
            page_number = request.GET.get('page')
            domain = paginator.get_page(page_number)
            ip = request.META.get('REMOTE_ADDR')
            res = requests.get('http://ip-api.com/json/'+ip)
            location_data_one = res.text
            geo = json.loads(location_data_one) 
            browser = request.META['HTTP_USER_AGENT']
            update = admin_update.objects.all().order_by('-id')[:5]
            return render(request, 'admin/account_info.html', {'blog': update, 'ip': ip, 'geo': geo, 'browser': browser,
                                                    'domains': domain, })
        
        else:
            return render(request, '404.html')

    else:
        return redirect('/admin')
    


def chuname(request):
    if request.user.is_authenticated:
        if request.user.is_staff:
            user = request.user
           
            if request.method == 'POST':
                fm = userchform(request.POST, instance=request.user)
                if fm.is_valid():
                    fm.save()
                    messages.success(request, 'Username Changed Successfully')
                    return redirect('/admin/account')

            else:
                fm = userchform(instance=request.user)
            return render(request, 'admin/chuname.html', {'form': fm})
        
        else:
            return render(request, '404.html')
    else:
        return redirect('/admin')
    

def chpass(request):
    if request.user.is_authenticated:
        if request.user.is_staff:
            user = request.user
            

            if request.method == 'POST':
                form = chpassform(user=request.user, data=request.POST)
                if form.is_valid():
                    form.save()
                    update_session_auth_hash(request, form.user)
                    messages.success(request, 'Password Changed Successfully')
                    return redirect('/admin/account')
            else:
                form = chpassform(user=request.user)
            return render(request, 'admin/chpass.html', {'form': form})
        
        else:
            return render(request, '404.html')

    else:
        return redirect('/admin')
    

def editprofile(request):
    if request.user.is_authenticated:
        if request.user.is_staff:
            user = request.user
           

            if request.method == 'POST':
                form = profileform(request.POST, instance=request.user)
                form1 = usrinfoform(request.POST, instance=request.user.userinfo)
                if form.is_valid() and form1.is_valid():
                    form.save()
                    form1.save()
                    messages.success(request, 'Profile Successfully Updated')
                    return redirect('/admin/account')

            else:
                form = profileform(instance=request.user)
                form1 = usrinfoform(instance=request.user.userinfo)
            return render(request, 'admin/editprofile.html', {'form': form, 'form1': form1})
        
        else:
            return render(request, '404.html')

    else:
        return redirect('/admin')
    


def admin_update_fun(request):

    if request.user.is_authenticated:
        if request.user.is_superuser:
           
            

            if request.method == 'POST':
                form = custom_admin_upadte(request.POST)
                if form.is_valid():
                    q = form.save(commit=False)
                    q.user = request.user
                    q.save()
                    messages.success(request, 'admin updated Successfully Created')
                    return redirect('/admin')

            else:
                form = custom_admin_upadte()
            return render(request, 'admin/admin_update.html', {'form': form})

        else:
            return render(request, '404.html')

    else:
        return redirect('/login')
    

def cloudflare(request):
    if request.user.is_authenticated:
        if request.user.is_superuser:
            try:
                query = cloudflare_api.objects.latest('id')
                get_zone_id = domains.objects.get(domain=query.main_domain_name).zone_id
                
                headers = {
                        'Content-Type': 'application/json',
                        'X-Auth-Email': query.email,
                        'X-Auth-Key': query.global_api_key
                    }
                
                url = f'https://api.cloudflare.com/client/v4/zones/{get_zone_id}'
                ssl_url = f"https://api.cloudflare.com/client/v4/zones/{get_zone_id}/settings/ssl"
                

                response = requests.get(url, headers=headers)
                response_for_ssl = requests.get(ssl_url, headers=headers)
                    # Handle the response
                if response.status_code == 200:
                    if response_for_ssl.status_code == 200:
                        get_ssl_status = response_for_ssl.json()['result']['value']

                    else:
                        return JsonResponse({'error': 'Failed to retrieve Cloudflare SSL/TLS encryption mode.'}, status=500)
                    
                    ns = response.json()['result']['name_servers']
                    return render(request, 'admin/cloudflare.html', {'cf': query, 'ssl_value': get_ssl_status, 'status': response.json()['result']['status'], 'name_servers1': ns[0], 'name_servers2': ns[1]})
                else:
                    return render(request, 'admin/db_cloudflare.html')
            except:
                return render(request, 'admin/db_cloudflare.html')
            
        else:
            return render(request, '404.html')
    
    else:
        return redirect('/login')
    

def cf_conf(request):
    if request.user.is_authenticated:
        if request.user.is_superuser:
            form_title = 'Cloudflare Authentigration'
            user = request.user
            
            if request.method == 'POST':
                form = cf_api_key(request.POST)
                if form.is_valid():
                    api_key = form.cleaned_data['global_api_key']
                    email = form.cleaned_data['email']
                     # Make a request to the Cloudflare API
                    headers = {
                        'Content-Type': 'application/json',
                        'X-Auth-Email': email,
                        'X-Auth-Key': api_key
                    }
                    url = 'https://api.cloudflare.com/client/v4/zones'

                    response = requests.get(url, headers=headers)
                    # Handle the response
                    if response.status_code == 200:

                        cloudflare_api.objects.update_or_create(defaults={'user': user, 'global_api_key': api_key, 'email': email})
                        messages.success(request, 'updated Successfully Created')
                        return redirect('/admin/cf/hostname')
                    
                    else:
                        messages.warning(request, "'error': 'Failed to fetch data from Cloudflare API'")
                        return render(request, 'admin/cf_config.html', {'head': form_title, 'form': form})
            else:
                try:
                    query = cloudflare_api.objects.latest('id')
                    form = cf_api_key(instance=query)
                except:
                    form = cf_api_key()
                
            return render(request, 'admin/cf_config.html', {'head': form_title, 'form': form})
        
        else:
            return render(request, '404.html')
        
    
    else:
        return redirect('login')
    



def cf_main_domain(request):
    if request.user.is_authenticated:
        if request.user.is_superuser:
            form_title = 'Cloudflare Authentigration'
            user = request.user
            try:
                query = cloudflare_api.objects.latest('id')

                if request.method == 'POST':
                    form = cf_hostname(request.POST)
                    if form.is_valid():
                        zone = form.cleaned_data['main_domain_name']
                        account_id = form.cleaned_data['account_id']
                        ip1 = form.cleaned_data['ip_address']
                        ip2 = form.cleaned_data['ip_address2']
                        # Make a request to the Cloudflare API
                        headers = {
                            'Content-Type': 'application/json',
                            'X-Auth-Email': query.email,
                            'X-Auth-Key': query.global_api_key
                        }
                        url = 'https://api.cloudflare.com/client/v4/zones'

                        # Data payload
                        data = {
                            "account": {
                                "id": account_id
                                },
                            "name": zone,
                            "type": "full"
                            }
                        
                        response = requests.post(url, headers=headers, json=data)
                        # Handle the response
                        if response.status_code == 200:
                            
                            zone_id = response.json()['result']['id']
                            dns_url = f"https://api.cloudflare.com/client/v4/zones/{zone_id}/dns_records"
                            
                            dns_records = [
                                        {
                                            "type": "A",
                                            "name": "@",
                                            "content": ip1,  # Replace with your first IP address
                                            "ttl": 1,
                                            "proxied": True
                                        },
                                        {
                                            "type": "A",
                                            "name": "@",
                                            "content": ip2,  # Replace with your second IP address
                                            "ttl": 1,
                                            "proxied": True
                                        },
                                        {
                                            "type": "CNAME",
                                            "name": "*",  # Replace with your wildcard domain
                                            "content": zone,  # Replace with the target domain
                                            "ttl": 1,
                                            "proxied": True
                                        }
                                        ]
                            for record in dns_records:
                                dns_response = requests.post(dns_url, json=record, headers=headers)
                            if dns_response.status_code == 200:
                                cloudflare_api.objects.update_or_create(defaults={'user': user, 'main_domain_name': zone, 'account_id': account_id, 'ip_address':
                                                                                            ip1, 'ip_address2': ip2})
                                                                                             
                                
                                domains.objects.create(user=user, domain=zone, Description=zone, zone_id=zone_id)
                                
                                #get_id = domains.objects.get(user=user, domain=zone).id
                                messages.success(request, 'updated Successfully Created')
                                return redirect(f'/admin/cf/')
                        
                            else:
                                cloudflare_api.objects.update_or_create(defaults={'user': user, 'main_domain_name': zone, 'account_id': account_id, 'ip_address':
                                                                                            ip1, 'ip_address2': ip2})
                                try:
                                    
                                    domains.objects.create(user=user, domain=zone, Description=zone, zone_id=zone_id)
                                except:
                                    domains.objects.filter(user=user, domain=zone).update(zone_id=zone_id)

                                messages.success(request, 'updated Successfully Created')
                                return redirect(f'/admin/cf/')
                            
                        
                        else:
                            messages.warning(request, "'error': 'Failed to fetch data from Cloudflare API Account id or domain already exists'")
                            return render(request, 'admin/cf_config.html', {'head': form_title, 'form': form})

                else:
                    form = cf_hostname(instance=query)
                return render(request, 'admin/cf_config.html', {'head': form_title, 'form': form})
            
            except:
                return redirect('/admin/cf/')
        else:
            return render(request, '404.html')
    else:
        return redirect('login')
    

#     # Get the Cloudflare API key from settings
#     # api_key = 'h8r23OIoFACJIC-dHX38-WzxW46C9vdV8SUbYBrT'
#     account_id = '88d3c5e4f04d8b52b5031850dacc825c'  # Replace with your Cloudflare account ID
#     zone_name = 'technicalbikash.com'  # Replace with your domain name
#     # ip_address = '192.0.2.1'  # Replace with the IP address you want to point to
     
#     # Make a request to the Cloudflare API
#     headers = {
#         'Content-Type': 'application/json',
#         'X-Auth-Email': 'pokhrelb9@gmail.com',
#         'X-Auth-Key': 'dc0016264d8cee9d65defc28026be46650ccc'
#     }
#     url = 'https://api.cloudflare.com/client/v4/zones'

#     # Data payload
#     data = {
#         "account": {
#             "id": account_id
#         },
#         "name": zone_name,
#         "type": "full"
#     }

#     response = requests.post(url, headers=headers, json=data)
#    # Handle the response
#     if response.status_code == 200:
#         #data = response.json()
#         # Process the data as needed
#         return HttpResponse(response)
#     else:
#          return JsonResponse({'error': 'Failed to fetch data from Cloudflare API'}, status=500)



# def ff(request):
#     domains.objects.create(user=request.user, domain='test.com', Description='')


def ssl_origin(request):
    if request.user.is_authenticated:
        if request.user.is_superuser:
            query = cloudflare_api.objects.latest('id')
            get_domain_query = domains.objects.get(domain=query.main_domain_name)
            get_zone_id = get_domain_query.zone_id
            
            doamin_name = get_domain_query.domain
            
            headers = {
                            'Content-Type': 'application/json',
                            'X-Auth-Email': query.email,
                            'X-Auth-Key': query.global_api_key
                        }
            url = f'https://api.cloudflare.com/client/v4/zones/{get_zone_id}/settings/ssl'
            

            data = {
                "value": "strict"
            }
            response = requests.patch(url, json=data, headers=headers)
            if response.status_code == 200:
                
                url_for_cert = f"https://api.cloudflare.com/client/v4/certificates"

                hostnames = [f"{doamin_name}", f"*.{doamin_name}"]
                csr_path = BASE_DIR / 'ssl/server.csr'
                # Read certificate and private key contents
                with open(csr_path, "r") as pem_file:
                    csr = pem_file.read()

                # data = {
                #         "certificate_request": csr,
                #         "hostnames": hostnames,
                #         "requested_validity": 5475  # Validity in days (15 years)
                #     }
                data = {
                        "csr": f"{csr}",
                        "hostnames": hostnames,
                        "request_type": "origin-rsa",
                        "requested_validity": 5475
                    }
                
                response2 = requests.post(url_for_cert, json=data, headers=headers)
                if response2.status_code == 200:
                    data = response2.json()
                    # Get the certificate value from the response JSON
                    certificate = data.get('result', {}).get('certificate')
                    # Save certificate to a PEM file
                    pem_filename = BASE_DIR / 'ssl/ssl-cert-snakeoil.pem'
                    with open(pem_filename, 'w') as pem_file:
                        pem_file.write(certificate)
                    messages.success(request, 'Certificate updated Successfully')
                    return redirect(f'/admin/cf/')
                
                else:
                    return JsonResponse({'error': 'Certificate not found in the response JSON.'}, status=400)
                
                #return redirect('/admin/cf/')
            else:
                return JsonResponse({'error': 'Failed to update SSL/TLS encryption mode.'}, status=500)

        else:
            return render(request, '404.html')
    else:
        return redirect('login')



# Generated by Django 5.0.2 on 2024-02-12 05:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0004_alter_domains_default'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='domains',
            name='default',
        ),
    ]



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Bigsansar",
    version="2.6.6",
    author="Bikash Pokhrel",
    author_email="bigsansaroffice@gmail.com",
    description="Build one in minutes with bigsansar - a visual site building tool!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pokhrelb9/bigsansar",
    project_urls={
        "Bug Tracker": "https://github.com/pokhrelb9/bigsansar/issues",
        "Documentations": "https://docs.bigsansar.com/"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux", 
        "Operating System :: OS Independent",
    ],

    packages=setuptools.find_packages(exclude=("www",
                                               "static",
                                               "ssl",
                                               "apache2"
                                               )),
    #packages=['bigsansar'],
    package_data={'': ['etc/*',
                       'templates/*',
                       'templates/*/*',
                                'static/*',
                                'static/ckeditor/ckeditor/plugins/youtube/*',
                                'static/ckeditor/ckeditor/plugins/youtube/*/*']},
                  
    include_package_data= True,

    entry_points={
        'console_scripts': [
                    'bigsansar = bigsansar.core:main',
                ],
    },

    python_requires=">=3.6",
    install_requires=['django', 'django-phonenumber-field[phonenumberslite]', 'django-ckeditor', 'requests', 'pillow', 'fontawesomefree'],
    keywords=['python', 'django host', 'bigsansar', 'django', 'django sites framework', 'django flatpages']
)



import json
import os
from django.shortcuts import redirect, render
from django.core.paginator import Paginator
import requests
from bigsansar.contrib.advance.models import SidebarSettings, admin_update
from bigsansar.contrib.sites.forms import create_domainform, customviewseditpage, customviewspageform, edit_domainform
from django.contrib import messages
from bigsansar.contrib.sites.models import default_domain, domains, pages
from django.utils.text import slugify
from django.core.exceptions import ValidationError

from www.settings import BASE_DIR

# Create your views here.


def manage_domain(request, id):
    if request.user.is_authenticated:
        if request.user.is_staff:

            try:
                ruser = request.user
                query = domains.objects.get(id=id, user=ruser)

            except:

                return render(request, '404.html')

            else:
                status = query.visitor
                if status == 0:
                    visitor = 'Deactive'

                else:
                    visitor = 'Active'
                user = request.user
                #domain = domains.objects.filter(user=user).order_by('-id')
                getpage = pages.objects.filter(domain=id).order_by('-id')
                paginator = Paginator(getpage, 5)
                page_number = request.GET.get('page')
                page = paginator.get_page(page_number)
                ip = request.META.get('REMOTE_ADDR')
                res = requests.get('http://ip-api.com/json/' + ip)
                location_data_one = res.text
                geo = json.loads(location_data_one)
                browser = request.META['HTTP_USER_AGENT']
                update = admin_update.objects.all().order_by('-id')[:5]
                
                return render(request, 'admin/domain_manage.html', {'visitcount': status, 'count': visitor, 'host': query,
                                                            'pages': page, 'ip': ip, 'geo': geo, 'browser': browser,
                                                            'blog': update})
            
        else:
            return render(request, '404.html')
    else:
        return redirect('/admin')
    


def edit_host(request, id):

    if request.user.is_authenticated:
        if request.user.is_staff:
            getdomains = domains.objects.filter(user=request.user).order_by('-id')

            try:
                user = request.user
                query = domains.objects.get(id=id, user=user)

            except:
                return render(request, '404.html')

            else:
                if request.method == 'POST':
                    form = edit_domainform(request.POST, instance=query)
                    if form.is_valid():
                        db = form.save(commit=False)
                        check_default = form.cleaned_data['primary_domaIn']
                        if check_default is True:
                            domains.objects.filter(user=request.user, primary_domaIn=True).update(primary_domaIn=False)
                            default_domain.objects.update_or_create(user=user, defaults={'domain': query})

                        else:
                            default_domain.objects.filter(user=user).delete()
                        
                        db.save()
                        messages.success(request, 'Domain Successfully Updated')
                        return redirect('/admin/domain/manage/' + id)
                else:
                    form = edit_domainform(instance=query)
                return render(request, 'admin/edit_domain.html', {'form': form})
            
        else:
            return render(request, '404.html')
    else:
        return redirect('/login')



def page_create(request, id):
    if request.user.is_authenticated:
        if request.user.is_staff:

            try:
                user = request.user
                get_host = domains.objects.get(pk=id, user=user)

            except:
                return render(request, '404.html')

            getdir = 'templates'
            if request.method == 'POST':
                form = customviewspageform(request.POST)
                if form.is_valid():

                    if pages.objects.filter(slug=slugify(form.cleaned_data.get('title')), domain=get_host):
                        messages.warning(request, 'page with url' + ' ' + slugify(
                            form.cleaned_data.get('title')) + ' ' + 'already exists for domain' + ' ' + get_host.domain)
                        form = customviewspageform
                        return render(request, 'admin/page_create.html', {'form': form})

                    query = form.save(commit=False)
                    query.domain = get_host
                    query.slug = slugify(form.cleaned_data.get('title'))
                    mkdir = getdir + '/' + str(get_host.domain)

                    try:
                        final_directory = os.path.join(BASE_DIR, mkdir)
                        if not os.path.exists(final_directory):
                            os.makedirs(final_directory)
                    except:
                        raise ValidationError(
                            _(final_directory + ' Directory not created "Permission Denied"'),
                        )

                    query.save()

                    messages.success(request, 'Pages Successfully Created')
                    return redirect('/admin/domain/manage/' + id)

            else:
                form = customviewspageform
            return render(request, 'admin/page_create.html', { 'form': form})
        
        else:
            return render(request, '404.html')

    else:
        return redirect('/admin')



def pages_edit(request, id, pagesid):
    if request.user.is_authenticated:
        if request.user.is_staff:

            getdir = 'templates'

            try:
                user = request.user
                get_host = domains.objects.get(pk=id, user=user)
                query = pages.objects.get(domain=id, id=pagesid)

            except:
                return render(request, '404.html')

            else:
                if request.method == 'POST':
                    form = customviewseditpage(request.POST, instance=query)
                    if form.is_valid():
                        db = form.save(commit=False)
                        db.domain = get_host

                        mkdir = getdir + '/' + str(get_host.domain)
                        full_path = os.path.join(BASE_DIR, mkdir)
                        createfile = full_path + '/' + query.slug + '.html'
                        get_content = form.cleaned_data.get('body')

                        f = open(createfile, "w", encoding="utf-8")
                        f.write(get_content)
                        f.close()
                        db.save()
                        messages.success(request, 'pages Successfully Updated')
                        return redirect('/admin/domain/manage/' + id)

                else:
                    form = customviewseditpage(instance=query)

                return render(request, 'admin/page_edit.html', {'form': form})
            
        else:
            return render(request, '404.html')
    else:

        return redirect('/admin')



def page_delete(request, id, pagesid):
    if request.user.is_authenticated:
        if request.user.is_staff:
            getdir = 'templates'

            try:
                user = request.user
                getdomain = domains.objects.get(pk=id, user=user)
                query = pages.objects.get(domain=id, id=pagesid)

            except:
                return render(request, '404.html')

            if request.method == 'POST':
                mkdir = getdir + '/' + str(getdomain.domain)
                createfile = mkdir + '/' + query.slug + '.html'

                if os.path.exists(createfile):
                    os.remove(createfile)
                    query.delete()
                else:
                    query.delete()

                messages.success(request, 'Pages Successfully Deleted')
                return redirect('/admin/domain/manage/' + id)
            return render(request, 'admin/page_delete.html', {'path': query, 'domain': getdomain})
        
        else:
            return render(request, '404.html')

    else:
        redirect('/admin')


# Create your views here.
def create_domain(request):
    if request.user.is_authenticated:
        if request.user.is_staff:
            user = request.user

            if request.method == 'POST':
                form = create_domainform(request.POST)

                if form.is_valid():
                    query = form.save(commit=False)
                    query.user = request.user
                    query.save()
                    messages.success(request, 'domain Successfully Created')
                    return redirect('/admin/account')

            else:
                form = create_domainform()
            return render(request, 'admin/create_domain.html', {'form': form})

        else:
            return render(request, '404.html')
    else:
        return redirect('/admin')
    




import getpass
import subprocess
import psycopg2

def connectposgres(database_name, username, password):

    con = psycopg2.connect(
    database="%s"  % (database_name) ,
    user="%s" % (username) ,
    password="%s" % (password) ,
    host="localhost",
    port= ""
    )

    cursor_obj = con.cursor()
    print(cursor_obj)

    if cursor_obj:
         # Open a file with access mode 'a'
            with open("www/settings.py", "a") as append_posgre:
                # Append 'postgres' at the end of file
                append_posgre.write(""
                                    "\n"
                                    "\n"
                                "DATABASES = {"
                                "\n"
                                "     'default': {"
                                "\n"
                                "         'ENGINE': 'django.db.backends.postgresql',"
                                "\n"
                                "         'NAME': '%s'," 
                                "\n"
                                "         'USER': '%s'," 
                                "\n"
                                "         'PASSWORD': '%s',"
                                "\n"
                                "         'HOST': 'localhost',"
                                "\n"
                                "         'PORT': '',"
                                "\n"
                                "     }"
                                 "\n"
                                  "}" % (database_name, username, password)
                                )
                                
                # Close the file
                append_posgre.close()

            print('postgresql: connection to server at "localhost" (127.0.0.1) successfully')
    else:
        print('connection to server at "localhost" (127.0.0.1) failed')

def execute_command(command, database_name=None, username=None, password=None):
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    output, error = process.communicate()
    if process.returncode != 0:
        print(f"Error: {error.decode('utf-8')}")
    else:
        print(output.decode('utf-8'))

        if database_name == None and username == None and password == None:
            pass
        else:

            # Open a file with access mode 'a'
            with open("www/settings.py", "a") as append_posgre:
                # Append 'postgres' at the end of file
                append_posgre.write(""
                                    "\n"
                                    "\n"
                                "DATABASES = {"
                                "\n"
                                "     'default': {"
                                "\n"
                                "         'ENGINE': 'django.db.backends.postgresql',"
                                "\n"
                                "         'NAME': '%s'," 
                                "\n"
                                "         'USER': '%s'," 
                                "\n"
                                "         'PASSWORD': '%s',"
                                "\n"
                                "         'HOST': 'localhost',"
                                "\n"
                                "         'PORT': '',"
                                "\n"
                                "     }"
                                 "\n"
                                  "}" % (database_name, username, password)
                                )
                                
                # Close the file
                append_posgre.close()


def create_database_and_user(database_name, username, password):

    
    create_db_command = f"sudo -u postgres psql -c 'CREATE DATABASE {database_name};'"
    create_user_command = f"sudo -u postgres psql -c \"CREATE USER {username} WITH PASSWORD '{password}';\""
    #alter_uff = f"sudo -u postgres psql -c \"ALTER ROLE {username} SET client_encoding TO 'utf8';\""
    grant_privileges_command = f"sudo -u postgres psql -c 'GRANT ALL PRIVILEGES ON DATABASE {database_name} TO {username};'"

    execute_command(create_db_command)
    execute_command(create_user_command)
    execute_command(grant_privileges_command, database_name, username, password)


print(' we are configuring postgresql for you ......................')

user_input = input('Do you have a already postgresql account (yes/no): ')

if user_input.lower() == 'yes':

    # Replace these with your desired values
    database_name = input("Enter the database name: ")
    username = input("Enter the username: ")
    password = getpass.getpass(prompt='Enter the password:')
    connectposgres(database_name, username, password)

elif user_input.lower() == 'no':
    # Replace these with your desired values
    database_name = input("Enter the database name: ")
    username = input("Enter the username: ")
    password = getpass.getpass(prompt='Enter the password:')

    create_database_and_user(database_name, username, password)

else:
    exit()
